﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Win32;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.IO;

namespace Mini.Controller
{
    public enum CacheMode { UseCache, OverwriteCache };

    public class PDFToBitmapConverter
    {
        private static string imageMagickBinPath;

        /// <summary>
        /// Returns the path to the ImageMagick binary folder
        /// </summary>
        public static string ImageMagickBinPath
        {
            get
            {
                if (imageMagickBinPath == null)
                {
                    RegistryKey reg_key = Registry.LocalMachine.OpenSubKey(@"Software\ImageMagick\Current");
                    imageMagickBinPath = (string)reg_key.GetValue("BinPath", false);
                }

                return imageMagickBinPath;
            }
        }

        /// <summary>
        /// Executes a process and returns all its standard output.
        /// </summary>
        /// <param name="exec_filename">The name of the executable</param>
        /// <param name="arguments">Arguments to pass to the executable</param>
        /// <returns>Standard output of the process</returns>
        protected static string SyncExecuteProcess(string filename, string arguments)
        {
            Process p = new Process();
            p.StartInfo.UseShellExecute = false;
            p.StartInfo.CreateNoWindow = true;
            p.StartInfo.RedirectStandardOutput = true;
            p.StartInfo.FileName = filename;
            p.StartInfo.Arguments = arguments;

            p.Start();
            string output = p.StandardOutput.ReadToEnd();
            p.WaitForExit();

            if (p.ExitCode != 0)
            {
                string mess = "Warning: " + p.StartInfo.FileName + " " + p.StartInfo.Arguments
                                + " terminated with exit status " + p.ExitCode.ToString();
                Console.Error.WriteLine(mess);
                throw new Exception(mess);
            }

            return output;
        }

        /// <summary>
        /// Converts <see cref="sourceFileName"/> into the format specified by 
        /// <see cref="destinationFileName"/>'s extension with given options
        /// </summary>
        /// <param name="sourceFileName">Full path to the original file</param>
        /// <param name="destinationFileName">Full path to the destination file</param>
        /// <param name="options">ImageMagick options</param>
        public static void SyncConvert(string sourceFileName, string destinationFileName, string options)
        {
            string exec_filename = ImageMagickBinPath + "\\" + "convert.exe";
            string arguments = options + " \"" + sourceFileName + "\" \"" + destinationFileName + "\"";

            SyncExecuteProcess(exec_filename, arguments);
        }

        /// <summary>
        /// Converts <see cref="sourceFileName"/> into the format specified by 
        /// <see cref="destinationFileName"/>'s extension with given options
        /// </summary>
        /// <param name="sourceFileName">Full path to the original file</param>
        /// <param name="destinationFileName">Full path to the destination file</param>
        public static void SyncConvert(string sourceFileName, string destinationFileName)
        {
            SyncConvert(sourceFileName, destinationFileName, "");
        }

        /// <summary>
        /// Gets number of pages in a pdf document.
        /// </summary>
        /// <param name="pdf_filename">The path to the pdf file</param>
        /// <returns>Number of pages</returns>
        public static int PdfNumberOfPages(string pdfFilename)
        {
            string exec_filename = ImageMagickBinPath + "\\" + "identify.exe";
            string arguments = "-format \"%n\" \"" + pdfFilename + "\"";
            string output = SyncExecuteProcess(exec_filename, arguments);

            string[] output_lines = Regex.Split(output, Environment.NewLine);

            return Int32.Parse(output_lines[0]);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pdfFilepath"></param>
        /// <param name="pageNum"></param>
        /// <param name="cacheMode"></param>
        /// <param name="resolution"></param>
        /// <returns>Full path to the resulting png file</returns>
        public static string ConvertPdfPageToPng(string pdfFilepath, int pageNum, CacheMode cacheMode, int resolution)
        {
            int ext_pos = pdfFilepath.LastIndexOf('.');
            int slash_pos = pdfFilepath.LastIndexOf(Path.DirectorySeparatorChar);

            string filename = pdfFilepath.Substring(slash_pos + 1, ext_pos - slash_pos - 1);
            string root_filename = MiniUtility.GetPathToStorageFolder(filename) + filename;

            string options = " -density " + resolution;
            string src_filename = pdfFilepath + "[" + pageNum + "]";
            string dst_filename = root_filename + pageNum + ".png";

            //TODO: fix this so it actually deletes the old file
            if (cacheMode == CacheMode.OverwriteCache)
            {
                if (!File.Exists(dst_filename))
                {
                    PDFToBitmapConverter.SyncConvert(src_filename, dst_filename, options);
                }

            }
            else if (cacheMode == CacheMode.UseCache)
            {
                //We are using the old cache. If it does not exist, then create it
                if (!File.Exists(dst_filename))
                {
                    PDFToBitmapConverter.SyncConvert(src_filename, dst_filename, options);
                }
            }

            return dst_filename;
        }
    }
}
